<?php

namespace App\Controller;

use App\Entity\History;
use App\Entity\Users;
use App\Entity\WebResetter;
use App\Repository\HistoryRepository;
use App\Repository\UsersRepository;
use App\Repository\WebResetterRepository;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

class ActionController extends AbstractController
{
    public function __construct(
        private readonly \App\Services\WebResetter $webResetter
    ) {
    }

    #[Route('/action/{action}', name: 'action', methods: ['POST'])]
    public function executeAction(
        Request $request,
        WebResetterRepository $webResetterRepository,
        ManagerRegistry $doctrine,
        string $action
    ): Response {
        if (!str_contains($request->getContentType(), 'json')) {
            return new JsonResponse([
                'success' => false,
                'message' => 'Content is not in JSON format'
            ], 415);
        }
        try {
            $requestBody = json_decode($request->getContent(), true, 512, JSON_THROW_ON_ERROR);
        } catch (\JsonException) {
            return new JsonResponse([
                'success' => false,
                'message' => 'Failed to decode JSON body'
            ], 415);
        }

        if (!in_array($action, ['power', 'reset', 'locate']) && !$this->isGranted('ROLE_ADMIN')) {
            return  new JsonResponse([
               'success' => false,
               'message' => 'The user is not allowed to execute this action'
            ], 403);
        }

        $webResetter = $webResetterRepository->findOneBy(['address' => $requestBody['address']]);
        if (!$webResetter) {
            return new JsonResponse([
                'success' => false,
                'message' => 'The given address doesn\'t exists'
            ]);
        }

        /** @var Users $user */
        $user =  $this->getUser();
        if (!in_array($this->getUser(), $webResetter->getUsers()->toArray()) && !$user->isAdminAccess()) {
            return new JsonResponse([
                'success' => false,
                'message' => 'You\'re not allowed to execute this action'
            ], 403);
        }

        $historyEntry = new History();
        $historyEntry->setUser($user);
        $historyEntry->setCommand($action);
        $historyEntry->setValue($requestBody["value"]);
        $historyEntry->setDevice($webResetter);
        $entityManager = $doctrine->getManager();
        $entityManager->persist($historyEntry);
        $entityManager->flush();

        $response = match ($action) {
            'power' => $this->webResetter->pressPowerButton($webResetter, $requestBody['value']),
            'reset' => $this->webResetter->pressResetButton($webResetter, $requestBody['value']),
            'locate' => $this->webResetter->locateLED($webResetter, $requestBody['value']),
            'shortDelay' => $this->webResetter->shortDelay($webResetter, $requestBody['value']),
            'longDelay' => $this->webResetter->longDelay($webResetter, $requestBody['value']),
            'address' => $this->webResetter->changeAddress($webResetter, $requestBody['value']),
            'resetController' => $this->webResetter->reset($webResetter),
            default => throw $this->createNotFoundException(),
        };

        if ($response !== true) {
            return new JsonResponse([
                'success' => false,
                'message' => 'Failed to communicate with the WebResetter: ' . $response
            ], 500);
        }

        return new JsonResponse([
            'success' => true,
            'message' => 'Action executed successfully',
        ]);
    }
}
