<?php

namespace App\Controller;

use App\Entity\Server;
use App\Entity\Users;
use App\Entity\WebResetter;
use App\Repository\ServerRepository;
use App\Repository\UsersRepository;
use App\Repository\WebResetterRepository;
use Doctrine\Persistence\ManagerRegistry;
use Exception;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;

class AdminController extends AbstractController
{
    #[Route('/admin/users', name: 'app_admin_users', methods: ['GET'])]
    public function displayUsers(UsersRepository $users): Response
    {
        return $this->render(
            'pages/admin/users.html.twig',
            [
                'users' => $users->findAll()
            ]
        );
    }

    #[Route('/admin/users', name: 'app_admin_users_edit', methods: ['POST'])]
    public function changeUser(
        UsersRepository $users,
        Request $request,
        UserPasswordHasherInterface $passwordHasher,
        ManagerRegistry $doctrine
    ): Response {
        $user = $users->findOneBy(['id' => $request->request->get('user')]);
        if (!$user) {
            $user = new Users();
            $doctrine->getManager()->persist($user);
        }

        $user->setName($request->request->getString('displayName'));
        if ($request->request->get('username') != $user->getUsername()) {
            if ($users->findOneBy(['username' => $request->request->get('username') ])) {
                $this->addFlash('error', [
                    'title' =>  $user->getUsername() ? 'Failed to change username' : 'Failed to create user',
                    'message' => 'The new username for this user is already in use, please try a different username.'
                ]);

                return $this->redirectToRoute('app_admin_users');
            }
            $user->setUsername($request->request->getString('username'));
        }
        $user->setAdminAccess($request->request->has('admin'));
        $user->setEnabled($request->request->has('enabled'));
        if ($request->request->get('password') !== '') {
            $user->setPassword(
                $passwordHasher->hashPassword($user, $request->request->getString('password'))
            );
        }
        $user->setGoogleAuthenticatorEnabled($request->request->has('totpEnabled'));

        $doctrine->getManager()->flush();
        $this->addFlash('notice', [
            'title' => 'User changed',
            'message' => 'The profile for ' . $user->getUsername() . ' was changed.'
        ]);

        return $this->redirectToRoute('app_admin_users');
    }

    #[Route('/admin/devices/{expandedDevice}', name: 'app_admin_devices', methods: ['GET'])]
    public function displayDevices(WebResetterRepository $webResetterRepository, int $expandedDevice = -1): Response
    {
        return $this->render('pages/admin/devices.html.twig', [
            'devices' => $webResetterRepository->findAll(),
            'expandedId' => $expandedDevice
        ]);
    }

    #[Route('/admin/devices', name: 'app_admin_devices_edit', methods: ['POST'])]
    public function editDevices(
        WebResetterRepository $webResetterRepository,
        ServerRepository $serverRepository,
        Request $request,
        ManagerRegistry $doctrine
    ): Response {
        $device = $webResetterRepository->findOneBy(['id' => $request->request->get('device')]);
        if (!$device) {
            $device = new WebResetter();
            $doctrine->getManager()->persist($device);
        }

        if ($request->request->get('address') != $device->getAddress()) {
            if ($webResetterRepository->findOneBy(['address' => $request->request->getString('address')])) {
                $this->addFlash('error', [
                    'title' =>  'Failed to save',
                    'message' => 'There is already a device with this address.'
                ]);

                return $this->redirectToRoute('app_admin_devices');
            }
        }

        $device->setName($request->request->getString('name'));
        $device->setNotes($request->request->getString('notes'));
        $device->setAddress($request->request->getInt('address'));
        $device->setIcon($request->request->getString('icon'));

        $server = $serverRepository->find($request->request->get('server'));
        if ($server) {
            $device->setServer($server);
        }

        $doctrine->getManager()->flush();

        return $this->redirectToRoute('app_admin_devices', [
            'expandedDevice' => $device->getId()
        ]);
    }

    #[Route('/admin/users/{userId}/permissions', methods: ['GET'], name: 'admin_users_permissions')]
    public function devicePermissionsGet(
        int $userId,
        UsersRepository $usersRepository,
        WebResetterRepository $webResetterRepository
    ): Response {
        $user = $usersRepository->find($userId);
        return $this->render('pages/admin/userPermissions.html.twig', [
           'user' => $user,
           'devices' => $webResetterRepository->findAll()
        ]);
    }

    #[Route('/admin/users/{userId}/permissions/save', methods: ['POST'], name: 'admin_users_permissions_save')]
    public function devicePermissionsSet(
        int $userId,
        UsersRepository $usersRepository,
        WebResetterRepository $webResetterRepository,
        ManagerRegistry $doctrine,
        Request $request
    ): Response {
        $user = $usersRepository->find($userId);
        foreach ($webResetterRepository->findAll() as $device) {
            if ($request->request->has('permission' . $device->getId())) {
                $device->addUser($user);
            } else {
                $device->removeUser($user);
            }
        }
        $doctrine->getManager()->flush();
        return $this->redirectToRoute('admin_users_permissions', ['userId' => $userId]);
    }

    #[Route('/admin/devices/settings/{deviceId}', methods: ['GET'], name: 'app_admin_devices_parameters')]
    public function deviceParametersGet(string $deviceId, WebResetterRepository $webResetterRepository): Response
    {
        $device = $webResetterRepository->find($deviceId);
        if (!$device) {
            throw new NotFoundHttpException('Can\'t find the device ID');
        }
        return $this->render('pages/admin/deviceSettings.html.twig', [
           'device' => $device
        ]);
    }

    #[Route('/admin/devices/locate/{deviceID}/{enableLocator}', name: 'app_admin_locator', methods: ['GET'])]
    public function locatorGet(
        int $deviceID,
        bool $enableLocator,
        WebResetterRepository $webResetterRepository,
        \App\Services\WebResetter $webResetter
    ): Response {
        $device = $webResetterRepository->find($deviceID);
        if (!$device) {
            throw $this->createNotFoundException();
        }

        try {
            $webResetter->locateLED($device, $enableLocator);
        } catch (Exception $e) {
            $this->addFlash('error', [
                'title' => 'Error setting locator LED',
                'message' => $e->getMessage()
            ]);
        }

        return $this->redirectToRoute('app_admin_devices', [
            'expandedDevice' => $deviceID
        ]);
    }

    #[Route('/admin/servers/{expandedDevice}', name: 'app_admin_servers', methods: ['GET'])]
    public function getServers(ServerRepository $serverRepository, int $expandedDevice = -1): Response
    {
        return $this->render('pages/admin/server.html.twig', [
           'servers' => $serverRepository->findAll(),
           'expandedId' => $expandedDevice
        ]);
    }

    #[Route('/admin/servers', name: 'app_admin_servers_edit', methods: ['POST'])]
    public function editServer(
        ServerRepository $serverRepository,
        Request $request,
        ManagerRegistry $doctrine
    ): Response {
        $server = $serverRepository->findOneBy(['id' => $request->request->get('server')]);
        if (!$server) {
            $server = new Server();
            $doctrine->getManager()->persist($server);
        }

        $server->setName($request->request->getString('name'));
        $server->setIpAddress($request->request->getString('ipaddress'));
        $server->setApikey($request->request->getString('apikey'));

        $doctrine->getManager()->flush();

        return $this->redirectToRoute('app_admin_servers', [
            'expandedDevice' => $server->getId()
        ]);
    }
}
